"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/use-fetch-frame.ts
var use_fetch_frame_exports = {};
__export(use_fetch_frame_exports, {
  useFetchFrame: () => useFetchFrame
});
module.exports = __toCommonJS(use_fetch_frame_exports);

// src/use-frame-stack.ts
var import_react = require("react");
function isParseResult(result) {
  return "status" in result;
}

// src/use-fetch-frame.ts
function computeDurationInSeconds(start, end) {
  return Number(((end.getTime() - start.getTime()) / 1e3).toFixed(2));
}
var defaultErrorHandler = (error) => {
  console.error(error);
};
function useFetchFrame({
  stackDispatch,
  specification,
  frameActionProxy,
  frameGetProxy,
  extraButtonRequestPayload,
  signFrameAction,
  onTransaction,
  homeframeUrl,
  onError = defaultErrorHandler
}) {
  async function handleFailedResponse({
    response,
    startTime,
    endTime,
    frameStackPendingItem
  }) {
    if (response.ok) {
      throw new TypeError(
        "handleFailedResponse called with a successful response"
      );
    }
    const responseBody = await getResponseBody(response);
    if (response.status >= 400 && response.status < 500) {
      if (frameStackPendingItem.request.method === "POST" && responseBody && typeof responseBody === "object" && "message" in responseBody && typeof responseBody.message === "string") {
        const stackItem = {
          ...frameStackPendingItem,
          responseStatus: response.status,
          speed: computeDurationInSeconds(startTime, endTime),
          status: "message",
          type: "error",
          message: responseBody.message,
          responseBody
        };
        stackDispatch({
          action: "DONE",
          pendingItem: frameStackPendingItem,
          item: stackItem
        });
        return;
      }
      console.error(
        `frames.js: The server returned an error but it does not contain message property. Status code: ${response.status}`,
        responseBody
      );
    }
    const requestError = new Error(
      `The server returned an error but it does not contain message property. Status code: ${response.status}`
    );
    pushRequestErrorToStack({
      frameStackPendingItem,
      requestError,
      responseStatus: response.status,
      startTime,
      endTime,
      responseBody
    });
  }
  function pushRequestErrorToStack({
    frameStackPendingItem,
    requestError,
    responseStatus,
    startTime,
    endTime,
    responseBody
  }) {
    stackDispatch({
      action: "REQUEST_ERROR",
      pendingItem: frameStackPendingItem,
      item: {
        request: frameStackPendingItem.request,
        requestDetails: frameStackPendingItem.requestDetails,
        timestamp: frameStackPendingItem.timestamp,
        url: frameStackPendingItem.url,
        responseStatus,
        requestError,
        speed: computeDurationInSeconds(startTime, endTime),
        status: "requestError",
        responseBody
      }
    });
    onError(requestError);
  }
  async function fetchGETRequest(request, shouldClear) {
    const startTime = /* @__PURE__ */ new Date();
    if (shouldClear) {
      stackDispatch({ action: "CLEAR" });
    }
    const frameStackPendingItem = {
      method: "GET",
      timestamp: startTime,
      request,
      status: "pending",
      requestDetails: {},
      url: request.url
    };
    stackDispatch({ action: "LOAD", item: frameStackPendingItem });
    const searchParams = new URLSearchParams({
      url: request.url,
      specification
    });
    const proxiedUrl = `${frameGetProxy}?${searchParams.toString()}`;
    const response = await tryCall(fetch(proxiedUrl, { method: "GET" }));
    const endTime = /* @__PURE__ */ new Date();
    if (response instanceof Response) {
      if (!response.ok) {
        await handleFailedResponse({
          response,
          startTime,
          endTime,
          frameStackPendingItem
        });
        return;
      }
      const loadedFrame = await response.json();
      stackDispatch({
        action: "DONE",
        pendingItem: frameStackPendingItem,
        item: {
          ...frameStackPendingItem,
          status: "done",
          frameResult: loadedFrame,
          speed: computeDurationInSeconds(startTime, endTime),
          responseStatus: response.status,
          responseBody: loadedFrame
        }
      });
      return;
    }
    pushRequestErrorToStack({
      frameStackPendingItem,
      endTime,
      startTime,
      responseBody: "none",
      responseStatus: 500,
      requestError: response
    });
  }
  async function fetchPOSTRequest(request, options, shouldClear) {
    const { address: _, ...requiredFrameContext } = request.signerStateActionContext.frameContext;
    const startTime = options?.startTime || /* @__PURE__ */ new Date();
    const frameStackPendingItem = {
      method: "POST",
      request,
      requestDetails: {
        // this will be filled afterwards
      },
      timestamp: startTime,
      status: "pending",
      url: request.frameButton.target || request.sourceFrame.postUrl || homeframeUrl || ""
      // this will be also filled after action is signed
    };
    if (shouldClear) {
      stackDispatch({ action: "CLEAR" });
    }
    if (!options) {
      stackDispatch({ action: "LOAD", item: frameStackPendingItem });
    }
    const signedDataOrError = await tryCall(
      signFrameAction(request.isDangerousSkipSigning, {
        ...request.signerStateActionContext,
        frameContext: requiredFrameContext
      })
    );
    if (signedDataOrError instanceof Error) {
      pushRequestErrorToStack({
        endTime: /* @__PURE__ */ new Date(),
        frameStackPendingItem,
        responseBody: "none",
        responseStatus: 500,
        startTime,
        requestError: signedDataOrError
      });
      return;
    }
    const { body, searchParams } = signedDataOrError;
    stackDispatch({
      action: "ADD_REQUEST_DETAILS",
      requestDetails: {
        body,
        searchParams
      },
      url: searchParams.get("postUrl") ?? "",
      pendingItem: frameStackPendingItem
    });
    searchParams.set("specification", specification);
    const proxiedUrl = `${frameActionProxy}?${searchParams.toString()}`;
    const response = await tryCall(
      fetch(proxiedUrl, {
        method: "POST",
        headers: {
          "Content-Type": "application/json"
        },
        body: JSON.stringify({
          ...extraButtonRequestPayload,
          ...body
        })
      })
    );
    const endTime = /* @__PURE__ */ new Date();
    if (response instanceof Response) {
      if (!response.ok) {
        await handleFailedResponse({
          response,
          startTime,
          endTime,
          frameStackPendingItem
        });
        return;
      }
      const responseData = await response.json();
      if ("location" in responseData) {
        const location = responseData.location;
        if (window.confirm(`You are about to be redirected to ${location}`)) {
          window.open(location, "_blank")?.focus();
        }
        return;
      }
      if ("message" in responseData) {
        stackDispatch({
          action: "DONE",
          pendingItem: frameStackPendingItem,
          item: {
            ...frameStackPendingItem,
            status: "message",
            message: responseData.message,
            type: "info",
            responseStatus: response.status,
            speed: computeDurationInSeconds(startTime, endTime),
            responseBody: responseData
          }
        });
        return;
      }
      if ("frameUrl" in responseData) {
        await fetchPOSTRequest(
          {
            // actions don't have source frame, fake it
            sourceFrame: {
              image: "",
              version: "vNext"
            },
            frameButton: {
              action: "post",
              label: "action",
              target: responseData.frameUrl
            },
            isDangerousSkipSigning: request.isDangerousSkipSigning,
            method: "POST",
            signerStateActionContext: {
              ...request.signerStateActionContext,
              buttonIndex: 1,
              frameButton: {
                action: "post",
                label: "action",
                target: responseData.frameUrl
              },
              target: responseData.frameUrl
            }
          },
          {
            pendingFrameStackItem: frameStackPendingItem,
            startTime
          }
        );
        return;
      }
      if (!isParseResult(responseData)) {
        const error = new Error(`The server returned an unexpected response.`);
        console.error(error);
        pushRequestErrorToStack({
          endTime,
          frameStackPendingItem,
          responseBody: responseData,
          responseStatus: 500,
          startTime,
          requestError: error
        });
        return;
      }
      stackDispatch({
        action: "DONE",
        pendingItem: options?.pendingFrameStackItem ?? frameStackPendingItem,
        item: {
          ...options?.pendingFrameStackItem ?? frameStackPendingItem,
          // update url to the one used to send the request, this handles transaction flow where pending item is created for transaction data request
          url: frameStackPendingItem.url,
          frameResult: responseData,
          status: "done",
          speed: computeDurationInSeconds(startTime, endTime),
          responseStatus: response.status,
          responseBody: responseData
        }
      });
      return;
    }
    pushRequestErrorToStack({
      endTime,
      frameStackPendingItem,
      responseBody: "none",
      responseStatus: 500,
      startTime,
      requestError: response
    });
  }
  async function fetchTransactionRequest(request, shouldClear) {
    const button = request.frameButton;
    if (button.action !== "tx") {
      throw new Error("Invalid frame button action, tx expected");
    }
    if (shouldClear) {
      stackDispatch({ action: "CLEAR" });
    }
    const startTime = /* @__PURE__ */ new Date();
    const frameStackPendingItem = {
      method: "POST",
      timestamp: startTime,
      status: "pending",
      request,
      // will be filled afterwards
      requestDetails: {},
      url: request.frameButton.target || request.sourceFrame.postUrl || homeframeUrl || ""
    };
    stackDispatch({ action: "LOAD", item: frameStackPendingItem });
    const signedTransactionDataActionOrError = await tryCall(
      signFrameAction(
        // for transaction data we always use signer, so skip signing is false here
        false,
        request.signerStateActionContext
      )
    );
    if (signedTransactionDataActionOrError instanceof Error) {
      pushRequestErrorToStack({
        endTime: /* @__PURE__ */ new Date(),
        frameStackPendingItem,
        responseBody: "none",
        responseStatus: 500,
        startTime,
        requestError: signedTransactionDataActionOrError
      });
      return;
    }
    stackDispatch({
      action: "ADD_REQUEST_DETAILS",
      requestDetails: {
        body: signedTransactionDataActionOrError.body,
        searchParams: signedTransactionDataActionOrError.searchParams
      },
      url: signedTransactionDataActionOrError.searchParams.get("postUrl") ?? "",
      pendingItem: frameStackPendingItem
    });
    signedTransactionDataActionOrError.searchParams.set(
      "specification",
      specification
    );
    const transactionDataProxiedUrl = `${frameActionProxy}?${signedTransactionDataActionOrError.searchParams.toString()}`;
    const transactionDataResponse = await tryCall(
      fetch(transactionDataProxiedUrl, {
        method: "POST",
        headers: {
          "Content-Type": "application/json"
        },
        body: JSON.stringify({
          ...extraButtonRequestPayload,
          ...signedTransactionDataActionOrError.body
        })
      })
    );
    const endTime = /* @__PURE__ */ new Date();
    if (!(transactionDataResponse instanceof Response)) {
      pushRequestErrorToStack({
        endTime,
        frameStackPendingItem,
        responseBody: "none",
        responseStatus: 500,
        startTime,
        requestError: transactionDataResponse
      });
      return;
    }
    if (!transactionDataResponse.ok) {
      await handleFailedResponse({
        response: transactionDataResponse,
        startTime,
        endTime,
        frameStackPendingItem
      });
      return;
    }
    const transactionData = await transactionDataResponse.json();
    const transactionIdOrError = await tryCall(
      onTransaction({
        frame: request.sourceFrame,
        frameButton: request.frameButton,
        transactionData
      }).then((transactionId) => {
        if (!transactionId) {
          return new Error("onTransaction did not return transaction id");
        }
        return transactionId;
      })
    );
    if (transactionIdOrError instanceof Error) {
      pushRequestErrorToStack({
        endTime,
        frameStackPendingItem,
        responseBody: "none",
        responseStatus: 500,
        startTime,
        requestError: transactionIdOrError
      });
      return;
    }
    await fetchPOSTRequest(
      {
        ...request,
        signerStateActionContext: {
          ...request.signerStateActionContext,
          // include transactionId in payload
          transactionId: transactionIdOrError,
          // override target so the the request is sent to proper endpoint
          target: button.post_url || request.sourceFrame.postUrl || button.target
        }
      },
      {
        pendingFrameStackItem: frameStackPendingItem,
        startTime
      }
    );
  }
  return (request, shouldClear = false) => {
    if (request.method === "GET") {
      return fetchGETRequest(request, shouldClear);
    }
    if (request.frameButton.action === "tx") {
      return fetchTransactionRequest(request, shouldClear);
    }
    return fetchPOSTRequest(request, void 0, shouldClear);
  };
}
async function tryCall(promise) {
  return promise.catch((e) => {
    if (e instanceof Error) {
      return e;
    }
    console.error(e);
    return new TypeError("Unexpected error, check the console for details");
  });
}
function getResponseBody(response) {
  if (response.headers.get("content-type")?.includes("/json")) {
    return response.clone().json();
  }
  return response.clone().text();
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useFetchFrame
});
