import { SupportedParsingSpecification } from 'frames.js';
import { v as FrameReducerActions, S as SignerStateActionContext, g as SignerStateInstance, O as OnTransactionFunc, j as FrameRequest } from './types-DXoU33KO.js';
import 'react';
import 'frames.js/frame-parsers';
import '@farcaster/core';

type UseFetchFrameOptions = {
    stackDispatch: React.Dispatch<FrameReducerActions>;
    specification: SupportedParsingSpecification;
    /**
     * URL or path to the frame proxy handling GET requests.
     */
    frameGetProxy: string;
    /**
     * URL or path to the frame proxy handling POST requests.
     */
    frameActionProxy: string;
    /**
     * Extra payload to be sent with the POST request.
     */
    extraButtonRequestPayload?: Record<string, unknown>;
    signFrameAction: (isDangerousSkipSigning: boolean, actionContext: SignerStateActionContext<any, any>) => ReturnType<SignerStateInstance["signFrameAction"]>;
    onTransaction: OnTransactionFunc;
    homeframeUrl: string | undefined | null;
    /**
     * This function can be used to customize how error is reported to the user.
     */
    onError?: (error: Error) => void;
};
type FetchFrameFn = (request: FrameRequest, shouldClear?: boolean) => Promise<void>;
declare function useFetchFrame({ stackDispatch, specification, frameActionProxy, frameGetProxy, extraButtonRequestPayload, signFrameAction, onTransaction, homeframeUrl, onError, }: UseFetchFrameOptions): FetchFrameFn;

export { useFetchFrame };
