"use strict";
"use client";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/use-frame.tsx
var use_frame_exports = {};
__export(use_frame_exports, {
  fallbackFrameContext: () => fallbackFrameContext,
  unsignedFrameAction: () => unsignedFrameAction,
  useFrame: () => useFrame
});
module.exports = __toCommonJS(use_frame_exports);
var import_react2 = require("react");
var import_core = require("@farcaster/core");

// src/use-frame-stack.ts
var import_react = require("react");
function isParseResult(result) {
  return "status" in result;
}
function framesStackReducer(state, action) {
  switch (action.action) {
    case "LOAD":
      return [action.item, ...state];
    case "ADD_REQUEST_DETAILS": {
      const index = state.findIndex(
        (item2) => item2.timestamp === action.pendingItem.timestamp
      );
      if (index === -1) {
        return state;
      }
      const item = state[index];
      if (!item || item.status !== "pending") {
        return state;
      }
      state[index] = {
        ...item,
        requestDetails: action.requestDetails,
        url: action.url
      };
      return state.slice();
    }
    case "DONE":
    case "REQUEST_ERROR": {
      const index = state.findIndex(
        (item) => item.timestamp === action.pendingItem.timestamp
      );
      if (index === -1) {
        return state;
      }
      state[index] = action.item;
      return state.slice();
    }
    case "RESET_INITIAL_FRAME": {
      const originalInitialFrame = state[0];
      const frame = isParseResult(action.resultOrFrame) ? action.resultOrFrame.frame : action.resultOrFrame;
      const shouldReset = !originalInitialFrame || originalInitialFrame.status === "done" && originalInitialFrame.frameResult.frame !== frame;
      if (shouldReset) {
        const frameResult = isParseResult(action.resultOrFrame) ? action.resultOrFrame : {
          status: "success",
          reports: {},
          frame: action.resultOrFrame
        };
        return [
          {
            request: {
              method: "GET",
              url: action.homeframeUrl ?? ""
            },
            url: action.homeframeUrl ?? "",
            requestDetails: {},
            responseStatus: 200,
            timestamp: /* @__PURE__ */ new Date(),
            speed: 0,
            frameResult,
            status: "done",
            // @todo should this be result or frame?
            responseBody: frameResult
          }
        ];
      }
      return state;
    }
    case "CLEAR":
      return [];
    default:
      return state;
  }
}
function useFrameStack({
  initialFrame,
  initialFrameUrl
}) {
  return (0, import_react.useReducer)(
    framesStackReducer,
    [initialFrame, initialFrameUrl],
    ([frame, frameUrl]) => {
      if (frame) {
        const frameResult = isParseResult(frame) ? frame : {
          reports: {},
          frame,
          status: "success"
        };
        return [
          {
            responseStatus: 200,
            responseBody: frameResult,
            timestamp: /* @__PURE__ */ new Date(),
            requestDetails: {},
            request: {
              method: "GET",
              url: frameUrl ?? ""
            },
            speed: 0,
            frameResult,
            status: "done",
            url: frameUrl ?? ""
          }
        ];
      } else if (frameUrl) {
        return [
          {
            method: "GET",
            request: {
              method: "GET",
              url: frameUrl
            },
            url: frameUrl,
            requestDetails: {},
            timestamp: /* @__PURE__ */ new Date(),
            status: "pending"
          }
        ];
      }
      return [];
    }
  );
}

// src/use-fetch-frame.ts
function computeDurationInSeconds(start, end) {
  return Number(((end.getTime() - start.getTime()) / 1e3).toFixed(2));
}
var defaultErrorHandler = (error) => {
  console.error(error);
};
function useFetchFrame({
  stackDispatch,
  specification,
  frameActionProxy,
  frameGetProxy,
  extraButtonRequestPayload,
  signFrameAction,
  onTransaction,
  homeframeUrl,
  onError = defaultErrorHandler
}) {
  async function handleFailedResponse({
    response,
    startTime,
    endTime,
    frameStackPendingItem
  }) {
    if (response.ok) {
      throw new TypeError(
        "handleFailedResponse called with a successful response"
      );
    }
    const responseBody = await getResponseBody(response);
    if (response.status >= 400 && response.status < 500) {
      if (frameStackPendingItem.request.method === "POST" && responseBody && typeof responseBody === "object" && "message" in responseBody && typeof responseBody.message === "string") {
        const stackItem = {
          ...frameStackPendingItem,
          responseStatus: response.status,
          speed: computeDurationInSeconds(startTime, endTime),
          status: "message",
          type: "error",
          message: responseBody.message,
          responseBody
        };
        stackDispatch({
          action: "DONE",
          pendingItem: frameStackPendingItem,
          item: stackItem
        });
        return;
      }
      console.error(
        `frames.js: The server returned an error but it does not contain message property. Status code: ${response.status}`,
        responseBody
      );
    }
    const requestError = new Error(
      `The server returned an error but it does not contain message property. Status code: ${response.status}`
    );
    pushRequestErrorToStack({
      frameStackPendingItem,
      requestError,
      responseStatus: response.status,
      startTime,
      endTime,
      responseBody
    });
  }
  function pushRequestErrorToStack({
    frameStackPendingItem,
    requestError,
    responseStatus,
    startTime,
    endTime,
    responseBody
  }) {
    stackDispatch({
      action: "REQUEST_ERROR",
      pendingItem: frameStackPendingItem,
      item: {
        request: frameStackPendingItem.request,
        requestDetails: frameStackPendingItem.requestDetails,
        timestamp: frameStackPendingItem.timestamp,
        url: frameStackPendingItem.url,
        responseStatus,
        requestError,
        speed: computeDurationInSeconds(startTime, endTime),
        status: "requestError",
        responseBody
      }
    });
    onError(requestError);
  }
  async function fetchGETRequest(request, shouldClear) {
    const startTime = /* @__PURE__ */ new Date();
    if (shouldClear) {
      stackDispatch({ action: "CLEAR" });
    }
    const frameStackPendingItem = {
      method: "GET",
      timestamp: startTime,
      request,
      status: "pending",
      requestDetails: {},
      url: request.url
    };
    stackDispatch({ action: "LOAD", item: frameStackPendingItem });
    const searchParams = new URLSearchParams({
      url: request.url,
      specification
    });
    const proxiedUrl = `${frameGetProxy}?${searchParams.toString()}`;
    const response = await tryCall(fetch(proxiedUrl, { method: "GET" }));
    const endTime = /* @__PURE__ */ new Date();
    if (response instanceof Response) {
      if (!response.ok) {
        await handleFailedResponse({
          response,
          startTime,
          endTime,
          frameStackPendingItem
        });
        return;
      }
      const loadedFrame = await response.json();
      stackDispatch({
        action: "DONE",
        pendingItem: frameStackPendingItem,
        item: {
          ...frameStackPendingItem,
          status: "done",
          frameResult: loadedFrame,
          speed: computeDurationInSeconds(startTime, endTime),
          responseStatus: response.status,
          responseBody: loadedFrame
        }
      });
      return;
    }
    pushRequestErrorToStack({
      frameStackPendingItem,
      endTime,
      startTime,
      responseBody: "none",
      responseStatus: 500,
      requestError: response
    });
  }
  async function fetchPOSTRequest(request, options, shouldClear) {
    const { address: _, ...requiredFrameContext } = request.signerStateActionContext.frameContext;
    const startTime = options?.startTime || /* @__PURE__ */ new Date();
    const frameStackPendingItem = {
      method: "POST",
      request,
      requestDetails: {
        // this will be filled afterwards
      },
      timestamp: startTime,
      status: "pending",
      url: request.frameButton.target || request.sourceFrame.postUrl || homeframeUrl || ""
      // this will be also filled after action is signed
    };
    if (shouldClear) {
      stackDispatch({ action: "CLEAR" });
    }
    if (!options) {
      stackDispatch({ action: "LOAD", item: frameStackPendingItem });
    }
    const signedDataOrError = await tryCall(
      signFrameAction(request.isDangerousSkipSigning, {
        ...request.signerStateActionContext,
        frameContext: requiredFrameContext
      })
    );
    if (signedDataOrError instanceof Error) {
      pushRequestErrorToStack({
        endTime: /* @__PURE__ */ new Date(),
        frameStackPendingItem,
        responseBody: "none",
        responseStatus: 500,
        startTime,
        requestError: signedDataOrError
      });
      return;
    }
    const { body, searchParams } = signedDataOrError;
    stackDispatch({
      action: "ADD_REQUEST_DETAILS",
      requestDetails: {
        body,
        searchParams
      },
      url: searchParams.get("postUrl") ?? "",
      pendingItem: frameStackPendingItem
    });
    searchParams.set("specification", specification);
    const proxiedUrl = `${frameActionProxy}?${searchParams.toString()}`;
    const response = await tryCall(
      fetch(proxiedUrl, {
        method: "POST",
        headers: {
          "Content-Type": "application/json"
        },
        body: JSON.stringify({
          ...extraButtonRequestPayload,
          ...body
        })
      })
    );
    const endTime = /* @__PURE__ */ new Date();
    if (response instanceof Response) {
      if (!response.ok) {
        await handleFailedResponse({
          response,
          startTime,
          endTime,
          frameStackPendingItem
        });
        return;
      }
      const responseData = await response.json();
      if ("location" in responseData) {
        const location = responseData.location;
        if (window.confirm(`You are about to be redirected to ${location}`)) {
          window.open(location, "_blank")?.focus();
        }
        return;
      }
      if ("message" in responseData) {
        stackDispatch({
          action: "DONE",
          pendingItem: frameStackPendingItem,
          item: {
            ...frameStackPendingItem,
            status: "message",
            message: responseData.message,
            type: "info",
            responseStatus: response.status,
            speed: computeDurationInSeconds(startTime, endTime),
            responseBody: responseData
          }
        });
        return;
      }
      if ("frameUrl" in responseData) {
        await fetchPOSTRequest(
          {
            // actions don't have source frame, fake it
            sourceFrame: {
              image: "",
              version: "vNext"
            },
            frameButton: {
              action: "post",
              label: "action",
              target: responseData.frameUrl
            },
            isDangerousSkipSigning: request.isDangerousSkipSigning,
            method: "POST",
            signerStateActionContext: {
              ...request.signerStateActionContext,
              buttonIndex: 1,
              frameButton: {
                action: "post",
                label: "action",
                target: responseData.frameUrl
              },
              target: responseData.frameUrl
            }
          },
          {
            pendingFrameStackItem: frameStackPendingItem,
            startTime
          }
        );
        return;
      }
      if (!isParseResult(responseData)) {
        const error = new Error(`The server returned an unexpected response.`);
        console.error(error);
        pushRequestErrorToStack({
          endTime,
          frameStackPendingItem,
          responseBody: responseData,
          responseStatus: 500,
          startTime,
          requestError: error
        });
        return;
      }
      stackDispatch({
        action: "DONE",
        pendingItem: options?.pendingFrameStackItem ?? frameStackPendingItem,
        item: {
          ...options?.pendingFrameStackItem ?? frameStackPendingItem,
          // update url to the one used to send the request, this handles transaction flow where pending item is created for transaction data request
          url: frameStackPendingItem.url,
          frameResult: responseData,
          status: "done",
          speed: computeDurationInSeconds(startTime, endTime),
          responseStatus: response.status,
          responseBody: responseData
        }
      });
      return;
    }
    pushRequestErrorToStack({
      endTime,
      frameStackPendingItem,
      responseBody: "none",
      responseStatus: 500,
      startTime,
      requestError: response
    });
  }
  async function fetchTransactionRequest(request, shouldClear) {
    const button = request.frameButton;
    if (button.action !== "tx") {
      throw new Error("Invalid frame button action, tx expected");
    }
    if (shouldClear) {
      stackDispatch({ action: "CLEAR" });
    }
    const startTime = /* @__PURE__ */ new Date();
    const frameStackPendingItem = {
      method: "POST",
      timestamp: startTime,
      status: "pending",
      request,
      // will be filled afterwards
      requestDetails: {},
      url: request.frameButton.target || request.sourceFrame.postUrl || homeframeUrl || ""
    };
    stackDispatch({ action: "LOAD", item: frameStackPendingItem });
    const signedTransactionDataActionOrError = await tryCall(
      signFrameAction(
        // for transaction data we always use signer, so skip signing is false here
        false,
        request.signerStateActionContext
      )
    );
    if (signedTransactionDataActionOrError instanceof Error) {
      pushRequestErrorToStack({
        endTime: /* @__PURE__ */ new Date(),
        frameStackPendingItem,
        responseBody: "none",
        responseStatus: 500,
        startTime,
        requestError: signedTransactionDataActionOrError
      });
      return;
    }
    stackDispatch({
      action: "ADD_REQUEST_DETAILS",
      requestDetails: {
        body: signedTransactionDataActionOrError.body,
        searchParams: signedTransactionDataActionOrError.searchParams
      },
      url: signedTransactionDataActionOrError.searchParams.get("postUrl") ?? "",
      pendingItem: frameStackPendingItem
    });
    signedTransactionDataActionOrError.searchParams.set(
      "specification",
      specification
    );
    const transactionDataProxiedUrl = `${frameActionProxy}?${signedTransactionDataActionOrError.searchParams.toString()}`;
    const transactionDataResponse = await tryCall(
      fetch(transactionDataProxiedUrl, {
        method: "POST",
        headers: {
          "Content-Type": "application/json"
        },
        body: JSON.stringify({
          ...extraButtonRequestPayload,
          ...signedTransactionDataActionOrError.body
        })
      })
    );
    const endTime = /* @__PURE__ */ new Date();
    if (!(transactionDataResponse instanceof Response)) {
      pushRequestErrorToStack({
        endTime,
        frameStackPendingItem,
        responseBody: "none",
        responseStatus: 500,
        startTime,
        requestError: transactionDataResponse
      });
      return;
    }
    if (!transactionDataResponse.ok) {
      await handleFailedResponse({
        response: transactionDataResponse,
        startTime,
        endTime,
        frameStackPendingItem
      });
      return;
    }
    const transactionData = await transactionDataResponse.json();
    const transactionIdOrError = await tryCall(
      onTransaction({
        frame: request.sourceFrame,
        frameButton: request.frameButton,
        transactionData
      }).then((transactionId) => {
        if (!transactionId) {
          return new Error("onTransaction did not return transaction id");
        }
        return transactionId;
      })
    );
    if (transactionIdOrError instanceof Error) {
      pushRequestErrorToStack({
        endTime,
        frameStackPendingItem,
        responseBody: "none",
        responseStatus: 500,
        startTime,
        requestError: transactionIdOrError
      });
      return;
    }
    await fetchPOSTRequest(
      {
        ...request,
        signerStateActionContext: {
          ...request.signerStateActionContext,
          // include transactionId in payload
          transactionId: transactionIdOrError,
          // override target so the the request is sent to proper endpoint
          target: button.post_url || request.sourceFrame.postUrl || button.target
        }
      },
      {
        pendingFrameStackItem: frameStackPendingItem,
        startTime
      }
    );
  }
  return (request, shouldClear = false) => {
    if (request.method === "GET") {
      return fetchGETRequest(request, shouldClear);
    }
    if (request.frameButton.action === "tx") {
      return fetchTransactionRequest(request, shouldClear);
    }
    return fetchPOSTRequest(request, void 0, shouldClear);
  };
}
async function tryCall(promise) {
  return promise.catch((e) => {
    if (e instanceof Error) {
      return e;
    }
    console.error(e);
    return new TypeError("Unexpected error, check the console for details");
  });
}
function getResponseBody(response) {
  if (response.headers.get("content-type")?.includes("/json")) {
    return response.clone().json();
  }
  return response.clone().text();
}

// src/use-frame.tsx
function onMintFallback({ target }) {
  window.alert(`Mint requested: ${target}`);
}
var unsignedFrameAction = async ({
  buttonIndex,
  frameContext,
  frameButton,
  state,
  target,
  inputText,
  url
}) => {
  const searchParams = new URLSearchParams({
    postType: frameButton.action,
    postUrl: target ?? ""
  });
  const isFarcaster = "castId" in frameContext;
  return {
    searchParams,
    body: {
      untrustedData: {
        url,
        timestamp: isFarcaster ? (0, import_core.getFarcasterTime)()._unsafeUnwrap() : Date.now(),
        ...isFarcaster ? { network: 1 } : {},
        buttonIndex,
        state,
        inputText,
        address: frameContext.connectedAddress,
        ...frameContext
      },
      trustedData: {
        messageBytes: Buffer.from("").toString("hex")
      }
    }
  };
};
async function onTransactionFallback({
  transactionData
}) {
  window.alert(
    `Requesting a transaction on chain with ID ${transactionData.chainId} with the following params: ${JSON.stringify(
      transactionData.params,
      null,
      2
    )}`
  );
  return null;
}
var fallbackFrameContext = {
  castId: {
    fid: 1,
    hash: "0x0000000000000000000000000000000000000000"
  },
  address: "0x0000000000000000000000000000000000000001"
};
function useFrame({
  homeframeUrl,
  frameContext,
  dangerousSkipSigning,
  onMint = onMintFallback,
  onTransaction = onTransactionFallback,
  connectedAddress,
  signerState,
  frame,
  /** Ex: /frames */
  frameActionProxy,
  /** Ex: /frames */
  frameGetProxy,
  extraButtonRequestPayload,
  specification = "farcaster",
  onError
}) {
  const [inputText, setInputText] = (0, import_react2.useState)("");
  const [framesStack, dispatch] = useFrameStack({
    initialFrame: frame,
    initialFrameUrl: homeframeUrl
  });
  const fetchFrame = useFetchFrame({
    frameActionProxy,
    frameGetProxy,
    onTransaction,
    signFrameAction(isDangerousSkipSigning, actionContext) {
      return isDangerousSkipSigning ? unsignedFrameAction(actionContext) : signerState.signFrameAction(actionContext);
    },
    specification,
    stackDispatch: dispatch,
    extraButtonRequestPayload,
    homeframeUrl,
    onError
  });
  const fetchFrameRef = (0, import_react2.useRef)(fetchFrame);
  fetchFrameRef.current = fetchFrame;
  (0, import_react2.useEffect)(() => {
    if (!frame && homeframeUrl) {
      fetchFrameRef.current(
        {
          url: homeframeUrl,
          method: "GET"
        },
        // tell the fetchFrame function to clear the stack because this is called only on initial render
        // and there could potentially be a pending object returned from SSR
        true
      ).catch((e) => {
        console.error(e);
      });
    } else if (frame) {
      dispatch({
        action: "RESET_INITIAL_FRAME",
        resultOrFrame: frame,
        homeframeUrl
      });
    }
  }, [frame, homeframeUrl, dispatch]);
  async function onButtonPress(currentFrame, frameButton, index, fetchFrameOverride = fetchFrame) {
    if (!signerState.hasSigner && !dangerousSkipSigning) {
      signerState.onSignerlessFramePress();
      return;
    }
    switch (frameButton.action) {
      case "link": {
        if (window.confirm(
          `You are about to be redirected to ${frameButton.target}`
        )) {
          parent.window.open(frameButton.target, "_blank");
        }
        break;
      }
      case "mint": {
        onMint({
          frameButton,
          target: frameButton.target,
          frame: currentFrame
        });
        break;
      }
      case "tx": {
        await onTransactionButton({
          frameButton,
          buttonIndex: index + 1,
          postInputText: currentFrame.inputText !== void 0 ? inputText : void 0,
          currentFrame
        });
        break;
      }
      case "post":
      case "post_redirect": {
        try {
          const target = frameButton.target || currentFrame.postUrl || homeframeUrl;
          if (!target) {
            throw new Error("missing target");
          }
          await onPostButton({
            currentFrame,
            frameButton,
            /** https://docs.farcaster.xyz/reference/frames/spec#handling-clicks
            
                    POST the packet to fc:frame:button:$idx:action:target if present
                    POST the packet to fc:frame:post_url if target was not present.
                    POST the packet to or the frame's embed URL if neither target nor action were present.
                    */
            target,
            buttonIndex: index + 1,
            postInputText: currentFrame.inputText !== void 0 ? inputText : void 0,
            state: currentFrame.state,
            fetchFrameOverride
          });
          setInputText("");
        } catch (err) {
          alert("error: check the console");
          console.error(err);
        }
        break;
      }
      default:
        throw new Error("Unrecognized frame button action");
    }
  }
  async function onPostButton({
    currentFrame,
    buttonIndex,
    postInputText,
    frameButton,
    target,
    state,
    transactionId,
    fetchFrameOverride
  }) {
    if (!dangerousSkipSigning && !signerState.hasSigner) {
      console.error("frames.js: missing required auth state");
      return;
    }
    if (!homeframeUrl) {
      console.error("frames.js: missing required value for post");
      return;
    }
    const _fetchFrame = fetchFrameOverride ?? fetchFrame;
    await _fetchFrame({
      frameButton,
      isDangerousSkipSigning: dangerousSkipSigning ?? false,
      method: "POST",
      signerStateActionContext: {
        inputText: postInputText,
        signer: signerState.signer ?? null,
        frameContext,
        url: homeframeUrl,
        target,
        frameButton,
        buttonIndex,
        state,
        transactionId
      },
      sourceFrame: currentFrame
    });
  }
  async function onTransactionButton({
    currentFrame,
    buttonIndex,
    postInputText,
    frameButton
  }) {
    if (!dangerousSkipSigning && !signerState.hasSigner) {
      console.error("frames.js: missing required auth state");
      return;
    }
    if (!homeframeUrl) {
      console.error("frames.js: missing required value for post");
      return;
    }
    await fetchFrame({
      frameButton,
      isDangerousSkipSigning: dangerousSkipSigning ?? false,
      method: "POST",
      signerStateActionContext: {
        inputText: postInputText,
        signer: signerState.signer ?? null,
        frameContext,
        address: connectedAddress,
        url: homeframeUrl,
        target: frameButton.target,
        frameButton,
        buttonIndex,
        state: currentFrame.state
      },
      sourceFrame: currentFrame
    });
  }
  return {
    inputText,
    setInputText,
    clearFrameStack: () => {
      dispatch({ action: "CLEAR" });
    },
    dispatchFrameStack: dispatch,
    onButtonPress,
    fetchFrame,
    homeframeUrl,
    framesStack,
    currentFrameStackItem: framesStack[0]
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  fallbackFrameContext,
  unsignedFrameAction,
  useFrame
});
